/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  cparam.c                                                */
/*                                                           */
/* Latest update: Jan 96 SS                                  */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>



void qqsort(d,m,n)
double *d;
int m;
int n;
{
  int l,r,s,i,j;

  double d2,temp;

  int sr[41];
  int sl[41];

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        } /* while */
        while (d[j] > d2) {
           j--;
        } /* while */
        if (i <= j) {
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           i++;
           j--;
        } /* if */
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort */


void quantiles(x,n,q,nq)
double *x;
int n;
double *q;
int nq;
{
   int iq,i,i0;
   double p,x0,x1;

   if (n == 1) {
      for (iq=1; iq<=(nq-1); iq++) {
         q[iq]=x[1];
      } /* for iq */
   }
   else {

      qqsort(x,1,n);

      for (i=1; i<=(nq-1); i++) {
         p=(double)(n)*(double)(i)/(double)(nq);
         i0=(int)(p);

         if  (i0 == 0) {
            x0=x[1];
            x1=0.5*(x[1]+x[2]);
         } 
         else if (i0 == (n-1)) {
            x0=0.5*(x[n-1]+x[n]);
            x1=x[n];
         }
         else {
            x0=0.5*(x[i0]+x[i0+1]);
            x1=0.5*(x[i0+1]+x[i0+2]);
         } /* if */

         q[i]=x0+(x1-x0)*(p-(double)(i0));

      } /* for i */

   } /* if */

} /* quantiles */
             
                     
      
                  

void rob_ave_std(x,n,ave,std)
double *x;
int n;
double *ave;
double *std;
{
   double q[6];

   quantiles(x,n,q,6);

   *ave=q[3];
   *std=(q[5]-q[1])/1.935;

} /* rob_ave_std */




int main(argc,argv) 
int argc;
char *argv[];

{
   FILE *infile;

   int n,i,j,k;

   double rmax2,rmin2,x,y,inte,rminn2,rmaxn2;
   double mean,median,ri,dx,dy,r2;
   double ssum,temp,sigma1,sigma2,intmin;

   double *xxx, *yyy,*rrr;


   if (argc < 2) {
      printf("Usage: %s file(x y int) intminr\n",argv[0]);
      exit(0);
   } /* if */ 

   intmin=atof(argv[2]);


   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   i=1;

   while (fscanf(infile,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      if (inte > intmin) {
         i++;
      } /* if */
   } /* while */

   n=i-1;


   xxx=(double *)calloc(n+1,sizeof(double));
   yyy=(double *)calloc(n+1,sizeof(double));
   rrr=(double *)calloc(n+1,sizeof(double));


   rewind(infile);

   i=1;

   while (fscanf(infile,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      if (inte > intmin) {
         xxx[i]=x;
         yyy[i]=y;
         i++;
      } /* if */
   } /* while */



   mean=0.0;
   rmin2=10000000.0;
   rmax2=0.0;

   for (i=1; i<=n; i++) {

      x=xxx[i];
      y=yyy[i]; 

      rminn2=10000000;

      for (j=1; j<=n; j++) {

         if (i != j) {
            dx=xxx[j]-x;
            dy=yyy[j]-y;

            r2=dx*dx+dy*dy;

            if (r2 < rminn2) {
               rminn2=r2;
            } /* if */

         } /* if */

        } /* for j */

        if (rminn2 > rmax2) {
           rmax2=rminn2;
        } /* if */

        if (rminn2 < rmin2) {
           rmin2=rminn2;
        } /* if */

        temp=sqrt(rminn2);
     
        mean=mean+temp;

        rrr[i]=temp;

    } /* for i */

    

    mean=mean/(double)(n);

    ssum=0.0;

    for (i=1; i<=n; i++) {
       temp=rrr[i]-mean;
       ssum=ssum+temp*temp;
    } /* for i */

    sigma1=sqrt(ssum/(double)(n-1));


    rob_ave_std(rrr,n,&median,&sigma2);


    printf("      n      d_min      d_max     d_mean    d_sigma"); 
    printf("  d_rob_mean d_rob_sigma\n");

    printf("%7d %10.3f %10.3f %10.3f %10.3f %11.3f %11.3f\n",
           n,sqrt(rmin2),sqrt(rmax2),mean,sigma1,median,sigma2);
    

                
} /* End */
