/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  fsortc.c                                                 */
/*                                                           */
/* Latest update: Nov 96 SS, July 98                         */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double timetic=(double)(CLOCKS_PER_SEC);

void qqsort(a,b,c,d,f,sr,sl,n)
float *a;
float *b;
float *c;
float *d;
float *f;
int *sr;
int *sl;
int n;
{
  int l,r,s,i,j;

  float m,temp;

  s=1;
  sl[s]=n;
  sr[s]=1;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     m=f[(l+r)/2];
     do {
        while (f[i] < m) {
           i--;
        } /* while */
        while (f[j] > m) {
           j++;
        } /* while */
        if (i >= j) {
           temp=a[i];
           a[i]=a[j];
           a[j]=temp;
           temp=b[i];
           b[i]=b[j];
           b[j]=temp;
           temp=c[i];
           c[i]=c[j];
           c[j]=temp;
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           temp=f[i];
           f[i]=f[j];
           f[j]=temp;
           i--;
           j++;
        } /* if */
     } while (i >= j);
     if (i > r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l > r);
  } while (s > 0);
   
} /* qqsort */


int main(argc,argv) 
int argc;
char *argv[];

{
   FILE *infile,*outfile;

   int i,j,k,n1,nn,n,logn;

   double r2,dx,dy,lt,mag;
   double rrej,rrej2;
   double rstep,rstep2,x1,y1;
   double xpos,ypos;

   float temp,inte1,sig1;

   float data[6];

   float *x,*y,*inte,*bg,*sig;

   int *sr,*sl;

   char *flag;

   clock_t time1,timediff1;


   if (argc < 4) {
      printf("Usage: %s findsfile.bin outfile.asc rrejr rstepr\n",argv[0]);
      exit(0);
   } /* if */

   rrej=atof(argv[3]);
   rstep=atof(argv[4]);


   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   rewind(infile);

   fseek(infile,0,SEEK_END);

   n=ftell(infile);

   n=n/(5*4);

   printf("%9d\n",n);

   x=(float *)calloc(n+1,sizeof(float));
   y=(float *)calloc(n+1,sizeof(float));
   inte=(float *)calloc(n+1,sizeof(float));
   bg=(float *)calloc(n+1,sizeof(float));
   sig=(float *)calloc(n+1,sizeof(float));

   flag=(char *)calloc(n+1,sizeof(char));

   logn=(int)(log((double)(n))/log(2.0)+10.5);

   sr=(int *)calloc(logn+1,sizeof(int));
   sl=(int *)calloc(logn+1,sizeof(int));

   rewind(infile);

   time1=clock();

   for (i=1; i<=n; i++) {
      fread(&data[1],sizeof(float),5,infile);
      x[i]=data[1];
      y[i]=data[2];
      inte[i]=data[3];
      bg[i]=data[4];
      sig[i]=data[5];
   } /* for i */

   fclose(infile);

   qqsort(x,y,bg,sig,inte,sr,sl,n);

   for (i=1; i<=n; i++) {
      flag[i]=0;
   } /* for i */

   outfile=fopen(argv[2],"w");

   
    lt=1.0/log(10.0);

    rrej2=rrej*rrej;
    rstep2=rstep*rstep;

    j=1;
    n1=1;

    while (n1 < n) {

       x1=(double)(x[n1]);
       y1=(double)(y[n1]);
       xpos=x1;
       ypos=y1;
       inte1=inte[n1];
       sig1=sig[n1];

       nn=n1+1;

       for (i=nn; i<=n; i++) {
           if (flag[i] == 0) {

              dx=(double)(x[i])-x1;
              dy=(double)(y[i])-y1;

              r2=dx*dx+dy*dy;

              if (r2 < rrej2) {
	         flag[i]=1;
	         if (r2 < rstep2) {
	            if (sig[i]  < sig1) {
		       xpos=(double)(x[i]);
		       ypos=(double)(y[i]);
		       sig1=sig[i];
		     } /* if */
	          } /* if */
               } /* if */ 

	  } /* if */     
       } /* for i */

       mag=-2.5*lt*log((double)(inte1));
       fprintf(outfile,"%6d %8.3f %8.3f %13.3f %8.3f %9.3f %12.6f\n",
       j,xpos,ypos,inte1,mag,bg[n1],sig[n1]);

       i=n1+1;
       while ((flag[i]==1) && (i < n)) {
          i++;
       } /* while */

       n1=i;

       j++;

   } /* while */ 

   fclose(outfile);

   timediff1=clock()-time1;

   printf("    nr    x-pos    y-pos           int      mag");
   printf("        bg        sigma\n");
   printf("\n");
   printf("    Printed to file: %s\n",argv[2]);

   printf("%9d\n",j-1);

   printf("%9.2f\n",(double)(timediff1)/timetic);
                
} /* End */
