/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  moffitf.c  SS June 92, Dec 97                            */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h> 
#include <unistd.h>
#include <stdlib.h>

double pi=3.14159265358979323;

int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *infile,*outfile;

   double *dpa,*dpat;

   double **a;
   double **at;

   double ata[6][6];
   double e[6][7];
   double b[6];
   double c[6];

   double x,y,z,sum,m,ekk,eps,max,t,fwhmtmp,w,x2,y2,sw,cw;
   double a0,b0,c0,d0,e0,f0,ap,bp,cp,dp,ep,fp,fwhma,fwhmb;
   double cv,sv,v,vd,ssum,msum,minmaxres,meanres,sigmares,res,maxtmp;
   double x0,y0,aaa,bbb,ggg,ddd,orig;
   double bg,lim,zim,beta,betamin,betamax,betaeps,stepd;
   double sigmamin,betatemp;

   int chimage,kmax2,xsize,ysize;
   int i,j,k,l,n,kwidth,korig,kmax,width,re,im;
   int step,ii;

   int xys[3];

   float fff[3];

   float **image;
   float *mpimage;

   if (argc<8) {
      printf("Usage: %s inimage.fff outimage.fff bgr limr ",argv[0]);
      printf("betaminr betamaxr betaepsr betastepi\n");
      exit(0);
   } /* if */


   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile=fopen(argv[2],"wb");

   
   bg=atof(argv[3]);
   lim=atof(argv[4]);
   betamin=atof(argv[5]);
   betamax=atof(argv[6]);
   betaeps=atof(argv[7]);
   step=atoi(argv[8]);


   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

   image=(float **)calloc(ysize,sizeof(float*));
   mpimage=(float *)calloc(xsize*ysize,sizeof(float));

   for (i=0; i<ysize; i++) {
      image[i]=mpimage;
      mpimage=mpimage+xsize;
   } /* for i */

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          fread(&fff[1],sizeof(float),1,infile);
	  image[i][j]=fff[1];
      } /* for j */
   } /* for i */


   width=xsize;
   orig=(double)(width-1)/2.0;


   n=5;

 /* LEAST SQARE FIT AtAc=B */

 /* A and At */

   k=0;
   for (i=0; i<width; i++) {
      for (j=0; j<width; j++){
         if (image[j][i]>=lim) {
            k++;
         } /* if */
      } /* for j */
    } /* for i */

    kmax=k-1;
    kmax2=k+1;


   a=(double **)calloc(kmax2,sizeof(double));
   at=(double **)calloc(6,sizeof(double));

   dpa=(double *)calloc(kmax2*6,sizeof(double));
   dpat=(double *)calloc(6*kmax2,sizeof(double));


   for (i=0; i<kmax2; i++) {
       a[i]=dpa;
       dpa=dpa+6;
   } /* for i */

   for (i=0; i<6; i++) {
       at[i]=dpat;
       dpat=dpat+kmax2;
   } /* for i */


   stepd=(betamax-betamin)/(double)(step);


while (stepd > betaeps) {

   sigmamin=1.0e30;

   for (ii=0; ii<=step; ii++) {

   beta=betamin+stepd*(double)(ii);

   k=0;
   for (i=0; i<width; i++) {
      for (j=0; j<width; j++){
         if (image[j][i]>=lim) {

            x=((double)(i)-orig);
            y=((double)(j)-orig);
            t=x*x; a[k][0]=t; at[0][k]=t;
            t=x*y; a[k][1]=t; at[1][k]=t;  
            t=y*y; a[k][2]=t; at[2][k]=t;
            t=x;   a[k][3]=t; at[3][k]=t;
            t=y;   a[k][4]=t; at[4][k]=t;
            t=1.0; a[k][5]=t; at[5][k]=t;
            k++;

         } /* if */
      } /* for j */
    } /* for i */


 /* AtA */

    for (i=0; i<=n; i++) {
       for (j=0; j<=n; j++) {
          sum=0.0;
          for (k=0; k<=kmax; k++) {
             sum=sum+at[i][k]*a[k][j];
          } /* for k */
          ata[i][j]=sum;
       } /* for j */
     } /* for i */       


 /* B */

    for (l=0; l<=n; l++) {
       sum=0.0;
       k=0;
       for (i=0; i<width; i++) {
          for (j=0; j<width; j++) {
            if (image[j][i]>=lim) {
               sum=sum+at[l][k]*(exp(-log((double)(image[j][i]-bg))/beta)-1.0);
               k++;
            } /* if */
          } /* for j */
       } /* for i */
       b[l]=sum;
    } /* for l */ 


 /* Gauss elimination AtAc=B */

    for (i=0; i<=n; i++) {
       for (j=0; j<=n; j++) {
          e[i][j]=ata[i][j];
       } /* for j */
    } /* for i */

    for (i=0; i<=n; i++) {
       e[i][n+1]=b[i];
    } /* for i */

    for (k=0; k<=n; k++) {
       ekk=e[k][k];
       for (j=k; j<=n+1; j++) {
          e[k][j]=e[k][j]/ekk;
       } /* for j */
       for (i=k+1; i<=n; i++) {
          for (j=k+1; j<=n+1; j++) {
             e[i][j]=e[i][j]-e[i][k]*e[k][j];
          } /* for j */ 
       } /* for i */
     } /* for k */

 /* Result: c */

     for (i=n; i>=0; i--) {
        c[i]=e[i][n+1];
        for (k=i+1; k<=n; k++) {
           c[i]=c[i]-e[i][k]*c[k];
        } /* for k */
     } /* for i */


 /* Resulting parameters */
 
   a0=c[0]; b0=c[1]; c0=c[2]; d0=c[3]; e0=c[4]; f0=c[5];

   v=atan2(b0,a0-c0)/2.0;

   vd=180.0/pi*v;

   cv=cos(v); sv=sin(v);

   ap=a0*cv*cv+b0*cv*sv+c0*sv*sv;
   bp=-2.0*a0*cv*sv+b0*(cv*cv-sv*sv)+2.0*c0*cv*sv;
   cp=a0*sv*sv-b0*cv*sv+c0*cv*cv;
   dp=d0*cv+e0*sv;
   ep=-d0*sv+e0*cv;
   fp=f0;

   fwhma=2.0*sqrt(dp*dp/(4.0*ap*ap)-
                  (1.0+fp-(1.0-dp*dp/(4.0*ap)+fp)*exp(log(2.0)/beta))/ap);
   fwhmb=2.0*sqrt(ep*ep/(4.0*cp*cp)-
                  (1.0+fp-(1.0-ep*ep/(4.0*cp)+fp)*exp(log(2.0)/beta))/cp);
  
   if (fwhmb>fwhma) {
      fwhmtmp=fwhma;
      fwhma=fwhmb;
      fwhmb=fwhmtmp;
   } /* if */

   aaa=a0;
   bbb=b0;
   ggg=c0;
   x0=(e0-2.0*c0*d0/b0)/(-b0+4.0*a0*c0/b0);
   y0=(d0-2.0*a0*e0/b0)/(-b0+4.0*a0*c0/b0);
   ddd=f0-aaa*x0*x0-bbb*x0*y0-ggg*y0*y0;

 /* Residuals: mean, sigma and minmax */

   ssum=0.0;
   msum=0.0;
   minmaxres=0.0;
   maxtmp=0.0;
 
   for (i=0; i<width; i++) {
      for (j=0; j<width; j++) {

         zim=image[j][i];

         if (zim >= lim) {
            
            x=((double)(i)-orig);
            y=((double)(j)-orig);

            
            z=exp(-beta*log(1.0+a0*x*x+b0*x*y+c0*y*y+d0*x+e0*y+f0));

            res=zim-bg-z;

            msum=msum+res; 
            ssum=ssum+res*res;

            if ((fabs(res)) > maxtmp) {
               maxtmp=fabs(res);
               minmaxres=res;
            } /* if */

          } /* if */
        } /* for j */
     } /* for i */

     meanres=msum/(kmax+1);

     sigmares=sqrt((ssum-msum*msum/(kmax+1))/kmax); 


     if (sigmares < sigmamin) {
        sigmamin=sigmares;
	betatemp=beta;
     } /* if */

     } /* for ii */

     betamin=betatemp-stepd;
     betamax=betatemp+stepd;
     stepd=(betamax-betamin)/(double)(step);


} /* while */     


   beta=betatemp; 

   k=0;
   for (i=0; i<width; i++) {
      for (j=0; j<width; j++){
         if (image[j][i]>=lim) {

            x=((double)(i)-orig);
            y=((double)(j)-orig);
            t=x*x; a[k][0]=t; at[0][k]=t;
            t=x*y; a[k][1]=t; at[1][k]=t;  
            t=y*y; a[k][2]=t; at[2][k]=t;
            t=x;   a[k][3]=t; at[3][k]=t;
            t=y;   a[k][4]=t; at[4][k]=t;
            t=1.0; a[k][5]=t; at[5][k]=t;
            k++;

         } /* if */
      } /* for j */
    } /* for i */


 /* AtA */

    for (i=0; i<=n; i++) {
       for (j=0; j<=n; j++) {
          sum=0.0;
          for (k=0; k<=kmax; k++) {
             sum=sum+at[i][k]*a[k][j];
          } /* for k */
          ata[i][j]=sum;
       } /* for j */
     } /* for i */       


 /* B */

    for (l=0; l<=n; l++) {
       sum=0.0;
       k=0;
       for (i=0; i<width; i++) {
          for (j=0; j<width; j++) {
            if (image[j][i]>=lim) {
               sum=sum+at[l][k]*(exp(-log((double)(image[j][i]-bg))/beta)-1.0);
               k++;
            } /* if */
          } /* for j */
       } /* for i */
       b[l]=sum;
    } /* for l */ 


 /* Gauss elimination AtAc=B */

    for (i=0; i<=n; i++) {
       for (j=0; j<=n; j++) {
          e[i][j]=ata[i][j];
       } /* for j */
    } /* for i */

    for (i=0; i<=n; i++) {
       e[i][n+1]=b[i];
    } /* for i */

    for (k=0; k<=n; k++) {
       ekk=e[k][k];
       for (j=k; j<=n+1; j++) {
          e[k][j]=e[k][j]/ekk;
       } /* for j */
       for (i=k+1; i<=n; i++) {
          for (j=k+1; j<=n+1; j++) {
             e[i][j]=e[i][j]-e[i][k]*e[k][j];
          } /* for j */ 
       } /* for i */
     } /* for k */

 /* Result: c */

     for (i=n; i>=0; i--) {
        c[i]=e[i][n+1];
        for (k=i+1; k<=n; k++) {
           c[i]=c[i]-e[i][k]*c[k];
        } /* for k */
     } /* for i */


 /* Resulting parameters */
 
   a0=c[0]; b0=c[1]; c0=c[2]; d0=c[3]; e0=c[4]; f0=c[5];

   v=atan2(b0,a0-c0)/2.0;

   vd=180.0/pi*v;

   cv=cos(v); sv=sin(v);

   ap=a0*cv*cv+b0*cv*sv+c0*sv*sv;
   bp=-2.0*a0*cv*sv+b0*(cv*cv-sv*sv)+2.0*c0*cv*sv;
   cp=a0*sv*sv-b0*cv*sv+c0*cv*cv;
   dp=d0*cv+e0*sv;
   ep=-d0*sv+e0*cv;
   fp=f0;

   fwhma=2.0*sqrt(dp*dp/(4.0*ap*ap)-
                  (1.0+fp-(1.0-dp*dp/(4.0*ap)+fp)*exp(log(2.0)/beta))/ap);
   fwhmb=2.0*sqrt(ep*ep/(4.0*cp*cp)-
                  (1.0+fp-(1.0-ep*ep/(4.0*cp)+fp)*exp(log(2.0)/beta))/cp);
  
   if (fwhmb>fwhma) {
      fwhmtmp=fwhma;
      fwhma=fwhmb;
      fwhmb=fwhmtmp;
   } /* if */

   aaa=a0;
   bbb=b0;
   ggg=c0;
   x0=(e0-2.0*c0*d0/b0)/(-b0+4.0*a0*c0/b0);
   y0=(d0-2.0*a0*e0/b0)/(-b0+4.0*a0*c0/b0);
   ddd=f0-aaa*x0*x0-bbb*x0*y0-ggg*y0*y0;

 /* Residuals: mean, sigma and minmax */

   ssum=0.0;
   msum=0.0;
   minmaxres=0.0;
   maxtmp=0.0;
 
   for (i=0; i<width; i++) {
      for (j=0; j<width; j++) {

         zim=image[j][i];

         if (zim >= lim) {
            
            x=((double)(i)-orig);
            y=((double)(j)-orig);

            
            z=exp(-beta*log(1.0+a0*x*x+b0*x*y+c0*y*y+d0*x+e0*y+f0));

            res=zim-bg-z;

            msum=msum+res; 
            ssum=ssum+res*res;

            if ((fabs(res)) > maxtmp) {
               maxtmp=fabs(res);
               minmaxres=res;
            } /* if */

          } /* if */
        } /* for j */
     } /* for i */

     meanres=msum/(kmax+1);

     sigmares=sqrt((ssum-msum*msum/(kmax+1))/kmax); 



 /* Write parameters */

      printf("\n");
      printf("Moffatfit\n\n");
      printf("lim=   %15.5f",lim);
      printf("   bg=    %15.5f",bg);
      printf("   beta=    %12.8f\n\n",beta);
      printf("Number of fitted points = %8d",(kmax+1));
 printf("%10d %10.6f\n\n",(width*width),(double)(kmax+1)/(double)(width*width));
      printf("mean=  %12.8f",meanres);
      printf("   sigma= %12.8f",sigmares);
      printf("   minmax=   %12.8f\n\n",minmaxres);
 printf("z=(1.0+a*(x-x0)^2+b*(x-x0)(y-y0)+c*(y-y0)^2+d)^-beta\n\n");
      printf("a=     %12.8f\n",aaa);
      printf("b=     %12.8f\n",bbb);
      printf("c=     %12.8f\n",ggg);
      printf("d=     %12.8f\n",ddd);
      printf("x0=    %12.8f\n",x0);
      printf("y0=    %12.8f\n\n",y0);
      printf("fwhma= %12.8f\n",fwhma);
      printf("fwhmb= %12.8f\n",fwhmb);
      printf("angel= %12.8f\n",vd);
      printf("\n");
  

 /* Change Image */

chimage=1; 

fwrite(&xys[1],sizeof(int),2,outfile);

if (chimage==1) {

   for (i=0; i<width; i++) {
      for (j=0; j<width; j++) {

         x=((double)(i)-orig);
         y=((double)(j)-orig);

         fff[1]=(float)(exp(-beta*log(1.0+a0*x*x+b0*x*y+c0*y*y+d0*x+e0*y+f0))+bg);

	 fwrite(&fff[1],sizeof(float),1,outfile);

      } /* for j */
   } /* for i */  

} /* if */

fclose(infile);
fclose(outfile);

} /* End */
