/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  aperf.c   SS Dec 94, July 98                             */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>


double pi=3.14159265358979323;



int main(argc,argv) 
int argc;
char *argv[];
{
   FILE *infile,*imfile,*outfile;
 
   int i,j,k,xi,yi,dmat,cn,xci,yci,n;
   int xi2,yi2,xi3,yi3,i2,j2,an,bn;
   int xsize,ysize;

   double r,v,vs,pi2,pi3,x,y;
   double ry,ri,rmag,xc,yc;
   double inte,mag,sum,temp;

   int xys[3];

   float fff[3];

   float **image;
   float *mpimage;

   short **mat;
   short *mpmat;
   

   int *bx,*by,*cx,*cy;


   if (argc<6) {
      printf("Usage: %s image.fff rbgmaxr rbgminr raperr file(x y) outfile\n",
             argv[0]);
      exit(0);
   } /* if */

   ry=atof(argv[2]);
   ri=atof(argv[3]);
   rmag=atof(argv[4]);


   if ((infile = fopen(argv[5],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   if ((imfile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile=fopen(argv[6],"w");

   fread(&xys[1],sizeof(int),2,imfile);

   xsize=xys[1];
   ysize=xys[2];

   image=(float **)calloc(ysize,sizeof(float*));
   mpimage=(float *)calloc(xsize*ysize,sizeof(float));

   for (i=0; i<ysize; i++) {
      image[i]=mpimage;
      mpimage=mpimage+xsize;
   } /* for i */

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          fread(&fff[1],sizeof(float),1,imfile);
	  image[i][j]=fff[1];
      } /* for j */
   } /* for i */
    
 
   dmat=2*(int)(ry+2.5);

   mat=(short **)calloc(dmat,sizeof(short*));
   mpmat=(short *)calloc(dmat*dmat,sizeof(short));

   for (i=0; i<dmat; i++) {
      mat[i]=mpmat;
      mpmat=mpmat+dmat;
   } /* for i */

   xc=(double)(int)(ry+2.0);
   yc=(double)(int)(ry+2.0);

   xci=(int)(xc);
   yci=(int)(yc);
   
   pi3=2.0*pi+pi/100.0;



   for (i=0; i<dmat; i++) {
      for (j=0; j<dmat; j++) {
         mat[i][j]=0;
      } /* for j */
   } /* for i */

   r=0.0;

   while (r <= rmag) {
      if (r < 0.5) {
         vs=pi;
      }
      else {
         vs=pi/(20.0*r);
      } /* if */
      v=0.0;
      while (v <= pi3) {
          xi=(int)(r*cos(v)+xc+0.5);
          yi=(int)(r*sin(v)+yc+0.5);
          mat[xi][yi]=1;
          v=v+vs;
       } /* while */
       r=r+0.5;
   } /* while */


   k=1;
   for (i=0; i<dmat; i++) {
      for (j=0; j<dmat; j++) {
         if (mat[i][j] == 1) {
            k++;
         } /* if */
      } /* for j */
   } /* for i */

  cn=k-1;

  cx = (int *)calloc(cn+2,sizeof(int));
  cy = (int *)calloc(cn+2,sizeof(int));
 
   k=1;
   for (i=0; i<dmat; i++) {
      for (j=0; j<dmat; j++) {
         if (mat[i][j] == 1) {
            cx[k]=i-xci;
            cy[k]=j-yci;
            k++;
         } /* if */
      } /* for j */
   } /* for i */



   for (i=0; i<dmat; i++) {
      for (j=0; j<dmat; j++) {
         mat[i][j]=0;
      } /* for j */
   } /* for i */

   r=ri;

   while (r <= ry) {
      if (r < 0.5) {
         vs=pi;
      }
      else {
         vs=pi/(20.0*r);
      } /* if */
      v=0.0;
      while (v <= pi3) {
          xi=(int)(r*cos(v)+xc+0.5);
          yi=(int)(r*sin(v)+yc+0.5);
          mat[xi][yi]=1;
          v=v+vs;
       } /* while */
       r=r+0.5;
   } /* while */


   k=1;
   for (i=0; i<dmat; i++) {
      for (j=0; j<dmat; j++) {
         if (mat[i][j] == 1) {
            k++;
         } /* if */
      } /* for j */
   } /* for i */

  bn=k-1;

  bx = (int *)calloc(bn+2,sizeof(int));
  by = (int *)calloc(bn+2,sizeof(int));

   k=1;
   for (i=0; i<dmat; i++) {
      for (j=0; j<dmat; j++) {
         if (mat[i][j] == 1) {
            bx[k]=i-xci;
            by[k]=j-yci;
            k++;
         } /* if */
      } /* for j */
   } /* for i */

  i=1;

  while(fscanf(infile,"%lf %lf",&x,&y) != EOF) {

     xi=(int)(x+0.5);
     yi=(int)(y+0.5);

     sum=0.0;
     j=0;

     for (k=1; k<=bn; k++) {
        xi2=bx[k]+xi;
        yi2=by[k]+yi;
        if ((xi2 >= 0) && (xi2 < xsize)) {
           if ((yi2 >= 0) && (yi2 < ysize)) {
              sum=sum+(double)(image[yi2][xi2]);
              j++;
           } /* if */
        } /* if */
     } /* for k */

     sum=sum/(double)(j);

     inte=0.0;
     
     for (j=1; j<=cn; j++) {
        xi2=cx[j]+xi;
        yi2=cy[j]+yi;
        if ((xi2 >= 0) && (xi2 < xsize)) {
           if ((yi2 >= 0) && (yi2 < ysize)) {
              temp=(double)(image[yi2][xi2])-sum;
              if (temp < 0.0) {
                 temp=0.0;
              } /* if */
              inte=inte+temp;
           } /* if */
        } /* if */
     } /* for j */

     if (inte < 1.0) {
        inte=1.0;
     } /* if */

     mag=-2.5*log(inte)/log(10.0);
  
     fprintf(outfile,"%6d %8.3f %8.3f %13.3f %9.3f %9.3f\n",
             i,x,y,inte,mag,sum);

     i++;

  } /* while */

  printf("    nr     xpos     ypos           int       mag        bg\n");
  printf("\n");
  printf("    Printed to file: %s\n",argv[6]);
  

  fclose(infile);
  fclose(imfile);
  fclose(outfile);


} /* End  */
