/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  fff2fits.c   SS Dec 97                                   */
/*  LINUX version                                            */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *outfile,*infile;
   
   int i,j,k,n,xsize,ysize,n2,n3;
   int n4,n33,ii,kk;
  
   unsigned char *d;

   unsigned char *ss;

   int xys[3];

   unsigned char d1[5];
   unsigned char d2[5];


   if (argc<3) {
      printf("Usage: %s image.fff image.fit\n",argv[0]);
      exit(0);
   } /* if */

   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile = fopen(argv[2],"wb");

   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

   ss=(unsigned char *)calloc(81,sizeof(unsigned char));

   sprintf(ss,"%s","SIMPLE  =                    T");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","BITPIX  =                  -32");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","NAXIS   =                    2");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s%6d","NAXIS1  =               ",xsize);

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s%6d","NAXIS2  =               ",ysize);

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","BSCALE  =       1.0000000000E0");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","BZERO   =       0.0000000000E0");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);

   sprintf(ss,"%s","END                           ");

   for (i=31; i<=81; i++) {
      ss[i]=' ';
   }

   fwrite(ss,sizeof(char),80,outfile);


   for (i=0; i<=81; i++) {
      ss[i]=' ';
   } 

   for (i=1; i<=28; i++) {
      fwrite(ss,sizeof(char),80,outfile);
   }


   printf("%6d %6d\n",xsize,ysize);

   n=xsize*ysize;

   n2=4*n;

   n3=n2 / 2880;

   n3=(n3+1)*2880;
 
   n33=n3/4;

   n4=(n3-n2)/4;

   d = (unsigned char *)calloc(4*n33+1,sizeof(unsigned char));

   kk=1;

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {

          fread(&d1[1],sizeof(unsigned char),4,infile);

	  for (ii=1; ii<=4; ii++) {
	     d2[5-ii]=d1[ii];
	  } /* for ii */

	  k=4*(xsize*(ysize-i-1)+j)+1;

          d[k]=d2[1];
          d[k+1]=d2[2];
          d[k+2]=d2[3];
          d[k+3]=d2[4];

	  kk=kk+4;

      } /* for j */
   } /* for i */

   for (i=1; i<=4*n4; i++) {
       d[kk]=0.0;
       kk++;
   } /* for i */

   fwrite(&d[1],sizeof(unsigned char),4*n33,outfile);

   fclose(outfile);
   fclose(infile);

} /* End  */
