/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  medianhf.c   SS July 92, Dec 97                          */
/*                                                           */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>

double timetic=(double)(CLOCKS_PER_SEC);

void qqsort(f,sr,sl,n)
float *f;
int *sr;
int *sl;
int n;
{
  int l,r,s,i,j;
  float temp,m;

  s=1;
  sl[s]=1;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     m=f[(l+r)/2];
     do {
        while (f[i] < m) {
           i++;
        } /* while */
        while (f[j] > m) {
           j--;
        } /* while */
        if (i <= j) {
           temp=f[i];
           f[i]=f[j];
           f[j]=temp;
           i++;
           j--;
        } /* if */
     } while (i <= j);
     if (i < r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     } /* if */
     r=j;
    } while (l < r);
  } while (s > 0);
   
} /* qqsort */

int main(argc,argv) 
int argc;
char *argv[];
{
  
   FILE *infile,*outfile;

   int i,j,i2,j2,k,l,bf,w,h;
   int xsize,ysize;
   int order,oo,s,ai,bi,aj,bj,m;

   double lim;

   int xys[3];

   float fff[3];

   float **image;
   float *mpimage;

   float *v;

   int *sr,*sl;

   clock_t time1,timediff;


   if (argc<4) {
      printf("Usage: %s inimage.fff outimage.fff orderi limitr\n",argv[0]);
      exit(0);
   } /* if */

  
   if ((infile = fopen(argv[1],"rb")) == NULL) {
      printf("File not found.\n");
      exit(0);
   } /* if */

   outfile=fopen(argv[2],"wb");

   order=atoi(argv[3]);
   lim=atof(argv[4]);

   oo=(int)(log((double)(order*order))/log(2.0))+11;

   v=(float *)calloc(order*order+1,sizeof(float));
   sr=(int *)calloc(oo+1,sizeof(int));
   sl=(int *)calloc(oo+1,sizeof(int));

   fread(&xys[1],sizeof(int),2,infile);

   xsize=xys[1];
   ysize=xys[2];

 
   image=(float **)calloc(ysize,sizeof(float*));
   mpimage=(float *)calloc(xsize*ysize,sizeof(float));

   for (i=0; i<ysize; i++) {
      image[i]=mpimage;
      mpimage=mpimage+xsize;
   } /* for i */

   for (i=0; i<ysize; i++) {
      for (j=0; j<xsize; j++) {
          fread(&fff[1],sizeof(float),1,infile);
	  image[i][j]=fff[1];
      } /* for j */
   } /* for i */

   s=(order-1)/2;

   time1=clock();

   fwrite(&xys[1],sizeof(int),2,outfile);

for (i=0; i<ysize; i++) {

   ai=s;
   bi=s;
   if (i<s) {
      ai=i;
   } /* if */
   if (i>(ysize-1-s)) {
      bi=ysize-1-i;
   } /* if */

   for (j=0; j<xsize; j++) {

      aj=s;
      bj=s;
      if (j<s) {
         aj=j;
      } /* if */
      if (j>(xsize-1-s)) {
         bj=xsize-1-j;
      } /* if */

      m=1;

      for (k=-ai; k<=bi; k++) {
         for (l=-aj; l<=bj; l++) {
            v[m]=image[i+k][j+l];
            m++;
         } /* for l */
      } /* for k */

      qqsort(v,sr,sl,m-1);

      if (fabs(image[i][j]-v[m/2]) > lim) {
         fff[1]=v[m/2];
      } 
      else {
         fff[1]=image[i][j];
      } /* if */

      fwrite(&fff[1],sizeof(float),1,outfile);

    } /* for j */
} /* for i */

   timediff=clock()-time1;

   printf("%8.2f\n",(double)(timediff)/timetic);

   fclose(infile);
   fclose(outfile);

} /* End  */
