/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  setpos.c                                                 */
/*                                                           */
/*  Latest update: Feb 94  SS                                */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double pi=3.14159265358979323;

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(idum)
long *idum;
{
   int j;
   long k;
   static long idum2=123456789;
   static long iy=0; 
   static long iv[NTAB];
   double temp;

   if (*idum <= 0) {
      if (-(*idum) < 1) {
         *idum=1;
      }
      else {
         *idum=-(*idum);
      } /* if */
      idum2=(*idum);
      for (j=NTAB+7; j>=0; j--) {
         k=(*idum)/IQ1;
	 *idum=IA1*(*idum-k*IQ1)-k*IR1;
	 if (*idum < 0) {
	    *idum += IM1;
	 } /* if */
	 if (j < NTAB) {
	    iv[j]=*idum;
	 } /* if */
      } /* for j */
      iy=iv[0];
   } /* if */
   k=(*idum)/IQ1;
   *idum=IA1*(*idum-k*IQ1)-k*IR1;
   if (*idum < 0) {
      *idum += IM1;
   } /* if */
   k=idum2/IQ2;
   idum2=IA2*(idum2-k*IQ2)-k*IR2;
   if (idum2 < 0) {
      idum2 += IM2;
   } /* if */
   j=iy/NDIV;
   iy=iv[j]-idum2;
   iv[j] = *idum;
   if (iy < 1) {
      iy += IMM1;
   } /* if */
   temp=AM*(double)(iy);
   if (temp > RNMX) {
      return(RNMX);
   }
   else {
      return(temp);
   } /* if */

} /* ran2 */

int main(argc,argv) 
int argc;
char *argv[];
{

   FILE *infile,*outfile;

   double lt,rnd,dx,dy,rmin,rmin2;
   double x,y,xc,yc,inte;

   int n1,ntot,nstep,sum,m,n,flag,flag2,sfi;
   int i,j,k,i2,xsize,ysize,nr,edge,xi,yi,seed,min,xyint;

   long idum;

   double *xv,*yv;


   if (argc<8) {
      printf("Usage: %s seedi xsizei ysizei xyinti(0/1) edgei rminr\n",argv[0]);
      printf("              lumfunc(n int) outfile\n");
      exit(0);
   } /* if */

  
seed=atoi(argv[1]);
xsize=atoi(argv[2]);
ysize=atoi(argv[3]);
xyint=atoi(argv[4]);
edge=atoi(argv[5]);
rmin=atof(argv[6]);

idum=-abs(seed)-1;


if ((infile = fopen(argv[7],"r")) == NULL) {
   printf("File not found.\n");
   exit(0);
} /* if */

outfile=fopen(argv[8],"w");

sum=0;

while (fscanf(infile,"%d %lf",&m,&inte) != EOF) {
   sum=sum+m;
} /* while */

fclose(infile);


xv = (double *)calloc(sum+3,sizeof(double));
yv = (double *)calloc(sum+3,sizeof(double));


rmin2=rmin*rmin;
k=1;

rewind(infile);

while (fscanf(infile,"%d %lf",&m,&inte) != EOF) {


for (j=1; j<=m; j++) {

   flag=1;

   while (flag != 0) {

      x=((double)(xsize-2*edge)*ran2(&idum)+(double)(edge));
      y=((double)(ysize-2*edge)*ran2(&idum)+(double)(edge));
      xi=(int)(x);
      yi=(int)(y);

      if (xyint == 1) {
         x=(double)(xi);
         y=(double)(yi);
      } /* if */

      i2=1;
      flag2=1;
      while ((flag2 != 0) && (i2 < k)) {
         dx=x-xv[i2];
         dy=y-yv[i2];
         if ((dx*dx+dy*dy) < rmin2) {
            flag2=0;
         } /* if */
         i2++;
      } /* while */

      if ((i2 == k) && (flag2 == 1)) {
        xv[k]=x;
        yv[k]=y;
        flag=0;
      } /* if */  

   } /* while */             


   fprintf(outfile,"%9.3f %9.3f %13.4f\n",x,y,inte);
   
   k++;

} /* for j */


} /* while */

fclose(infile);
fclose(outfile);


} /* End */
