/***************************************************************************/
/*                                                                         */
/*                                                                         */
/* SSPHOT A Photometry Package                                             */
/* ===========================                                             */
/*                                                                         */
/* This C-program is part of the SSPHOT photometry package.                */
/* See the links below for more information and documentation.             */
/*                                                                         */
/* Author                                                                  */
/* ======                                                                  */
/*                                                                         */
/* Stefan Spaennare, Lund, Sweden                                          */
/*                                                                         */
/* E-mail: stefan@spaennare.se                                             */
/*                                                                         */
/* Home page:                                                              */
/*                                                                         */
/* http://www.spaennare.se/index.html                                      */
/*                                                                         */
/* Please download the package and the documentation from the web-page:    */
/*                                                                         */
/* http://www.spaennare.se/ssphot.html                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

/********************************************************************************/
/*                                                                              */
/* Notice                                                                       */
/* ======                                                                       */
/*                                                                              */
/* I make no warranties that this program is (1) free of errors, (2) consistent */
/* with any standard merchantability, or (3) meeting the requirements of a      */
/* particular application. This software shall not, partly or as a whole,       */
/* participate in a process, whose outcome can result in injury to a person or  */
/* loss of property. It is solely designed for analytical work. Permission to   */
/* use, copy and distribute is hereby granted without fee, providing that the   */
/* header above including this notice appears in all copies.                    */
/*                                                                              */
/*                                                            Stefan Spaennare  */
/*                                                                              */
/********************************************************************************/

/*************************************************************/
/*  sscomp.c                                                 */
/*                                                           */
/* Latest update: May 95 SS                                  */
/*************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

double timetic=(double)(CLOCKS_PER_SEC);


void qqsort(b,c,d,sr,sl,m,n)
double *b;
double *c;
double *d;
int *sr;
int *sl;
int m;
int n;
{
  int l,r,s,i,j;

  double d2,temp;

  s=1;
  sl[s]=m;
  sr[s]=n;

  do {
     l=sl[s];
     r=sr[s];
     s--;
    do { 
     i=l;
     j=r;
     d2=d[(l+r)/2];
     do {
        while (d[i] < d2) {
           i++;
        }
        while (d[j] > d2) {
           j--;
        }
        if (i <= j) {
           temp=b[i];
           b[i]=b[j];
           b[j]=temp;
           temp=c[i];
           c[i]=c[j];
           c[j]=temp;
           temp=d[i];
           d[i]=d[j];
           d[j]=temp;
           i++;
           j--;
        }
     } while (i<=j);
     if (i<r) {
        s++;
        sl[s]=i;
        sr[s]=r;
     }
     r=j;
    } while (l<r);
  } while (s > 0);
   
} /* qqsort */


int main(argc,argv) 
int argc;
char *argv[];

{
   FILE *infile1,*infile2,*outfile;

   int i,j,n1,n2,n,ls,p,kk,n22;

   double rmax,x,y,dx,dy,inte,rmax2,mind;
   double dd,r2,inte1,inte2,intmin,dmag,lt;

   double *x1,*y1,*int1,*x2,*y2,*int2;
 
   int *sr,*sl; 

   clock_t time1,timediff1;


   if (argc != 6) {
      printf("Usage: %s file1(x y int) file2(x y int) rmaxr intminr outfile\n",argv[0]);
      exit(0);
   } /* if */ 


   rmax=atof(argv[3]);
   intmin=atof(argv[4]);

   if (intmin < 1.0) {
      intmin=1.0;
   } /* if */


   if ((infile1 = fopen(argv[1],"r")) == NULL) {
      printf("File %s not found.\n",argv[1]);
      exit(0);
   } /* if */

   if ((infile2 = fopen(argv[2],"r")) == NULL) {
      printf("File %s not found.\n",argv[2]);
      exit(0);
   } /* if */

   outfile=fopen(argv[5],"w");

   i=1;

   while (fscanf(infile1,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      i++;
   } /* while */

   n1=i-1;

   i=1;

   while (fscanf(infile2,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      i++;
   } /* while */

   n2=i-1;


   n=n1;
  
   if (n2 > n1) {
      n=n2;
   } /* if */

   ls=(int)(2.0*log((double)(n))/log(2.0)+0.5)+10;

   x1=(double *)calloc(n1+2,sizeof(double));
   y1=(double *)calloc(n1+2,sizeof(double));
   int1=(double *)calloc(n1+2,sizeof(double));

   x2=(double *)calloc(n2+2,sizeof(double));
   y2=(double *)calloc(n2+2,sizeof(double));
   int2=(double *)calloc(n2+2,sizeof(double));

   sr=(int *)calloc(ls+1,sizeof(int));
   sl=(int *)calloc(ls+1,sizeof(int));


   rewind(infile1);

   i=1;

   while (fscanf(infile1,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      x1[i]=x;
      y1[i]=y;
      int1[i]=inte;
      i++;
   } /* while */

   rewind(infile2);

   i=1;

   while (fscanf(infile2,"%lf %lf %lf",&x,&y,&inte) != EOF) {
      x2[i]=x;
      y2[i]=y;
      int2[i]=inte;
      i++;
   } /* while */

   qqsort(x1,y1,int1,sr,sl,1,n1);

   qqsort(x2,y2,int2,sr,sl,1,n2);


   rmax2=rmax*rmax;

   lt=1.0/log(10.0);

   time1=clock();


   n22=n2;
   kk=0;


   for (i=n1; i>=1; i--) {

       x=x1[i];
       y=y1[i];
       inte1=int1[i];

       mind=100000.0;

       p=0;

       for (j=n22; j>=1; j--) {
           
          dx=x-x2[j];
          dy=y-y2[j];
          inte2=int2[j];

          r2=dx*dx+dy*dy;
 
          if (r2 < rmax2) {
             if ((inte1 >= intmin) && (inte2 >= intmin)) {
                dd=fabs(log(inte1/inte2));
             }
             else {
                dd=1000000.0;
             } /* if */ 

             if (dd < mind) {
                mind=dd;
                p=j;
             } /* if */
          } /* if */

       } /* for j */


       if (p > 0) {

          dmag=-2.5*lt*log(int2[p]/inte1);
          fprintf(outfile,"%9.3f %9.3f %10.1f %9.3f %9.3f %10.1f %7.3f %7.3f %7.3f\n",
	          x,y,inte1,x2[p],y2[p],int2[p],x2[p]-x,y2[p]-y,dmag);

	  kk++;

	  for (j=p; j<n22; j++) {
	    x2[j]=x2[j+1];
	    y2[j]=y2[j+1];
	    int2[j]=int2[j+1];
	  } /* for j */

          n22--;

       } /* if */

   } /* for i */

   timediff1=clock()-time1;

   printf("       x1        y1       int1");
   printf("        x2        y2       int2");
   printf("      dx      dy    dmag\n");
   printf("\n");
   printf("       Printed to file: %s\n",argv[5]);
   printf("       Accepted stars: %8d\n",kk);

   printf("%12.2f\n",(double)(timediff1)/timetic);

   fclose(infile1);
   fclose(infile2);
   fclose(outfile);
                
} /* End */
